      Program EAGW
C     ************

C     Actual evapotranspiration and groundwater recharge are
C     calculated either from the Bagrov equation or from a 
C     statistic-based hydropedotransfer function depending on
C     site conditions.
C     Autor: Dr. K. Bohne, klaus.bohne@uni-rostock.de

      implicit real*8 (a-h,o-z)
      dimension ARAIN(10),AEpot(10)

      common /soils/ soilp(170),flux(68)
      external VANGEN, Simpson, Newton, Bagrov

      print *,'    '
      print *,' ***************************************************'  
      print *,' Prediction of average long-term actual             '
      print *,' evapotranspiration and groundwater recharge of     '
      print *,' grass-covered soils on sites without fast runoff   '
      print *,' ***************************************************'  
      print *,'  '
   1  continue
      print *,' Long-term average annual precipitation '
      print *,' and potential evapotranspiration, cm '
      read  (*,*) Rain, Epot
      print *,' Long-term average annual summer precipitation'
      print *,' and potential summer evapotranspiration, cm '
      read (*,*) SRain, SEPOT
     
C     coefficient of simultaneity

      print *, ' Is coefficient of simultaneity known? (0,1) '
      ny=0
      read (*,*) ny
      if (ny == 0) then
        print *,
     # 'Input monthly average rain and pot. Evapotranspiration, cm'  
        denum=0.
        ZL=0.
        do 7 i=4,9
         print *,' Month ', i
         print *,'Rain, Epot   '
         read (*,*) ARAIN(i), AEpot(i)
         denum= denum+AEpot(i)
	 Zero=0.
         ZL=ZL+AMAX1(Zero,AEpot(i)-ARAIN(i))
    7   continue
        Cs=ZL/denum
        print *,'    '
        write(*,700) Cs
  700   format('Cs= ',F6.3)
      else  
        print *,' coefficient of simultaneity '
        read (*,*) Cs
      end if
    2 continue
      print *,' Root zone depth and depth to groundwater (cm):' 
      read (*,*) RD, GW
     
   5  continue
      print *,' Ss  1      Su2 8     Lt3 15    Ut3  22     Ts2  29 ' 
      print *,' Sl2 2      Su3 9     Lts 16    Ut4  23     Ts3  30 '
      print *,' Sl3 3      Su4 10    Lu  17    Tt   24     Ts4  31 '
      print *,' Sl4 4      Ls2 11    Uu  18    Tl   25     fS   32 '
      print *,' Slu 5      Ls3 12    Uls 19    Tu2  26     mS   33 '
      print *,' St2 6      Ls4 13    Us  20    Tu3  27     gS   34 '
      print *,' St3 7      Ut2 21    Ut2 21    Tu4  28 '
      print *,'  '

      print *,' Soil class number (see program description): '
      print *,' To use other soil parameters, type 0 (zero)  '
      read (*,*) ksoil
      if ( ksoil .gt. 34) then
          print *,' wrong number!'
          goto 5
      end if
      print *,'   '
      if (ksoil == 0) then
        print *,' Model van Genuchten /Mualem '
        print *,' theta-r, theta-s '
        read (*,*) thr, ths
        print *,' alpha (1/cm), n '
        read (*,*) alpha,gn
        print *,' Parameters p1 and p2 to estimate qmax '
        read (*,*) p1,p2
      else
C     -----  get standard soil hydraulic parameters -----
        i =5*(ksoil-1)+1
        thr=  soilp(i+1)
        ths=  soilp(i+2)
        alpha=soilp(i+3)
        gn=   soilp(i+4)
        gm=   1.-1./gn
        i = 2*(ksoil-1)+1
        p1=   flux(i)
        p2=   flux(i+1)
      endif

C     ---   maximum plant available soil water storage  ---

      h=63.
      Call VANGEN (thr, ths, alpha, gn,h,fc)
      h=15800.
      Call VANGEN (thr, ths, alpha, gn,h,pwp)
      Wa=RD*(fc-pwp)
      write (*,704) fc,pwp,Wa
  704 format(/,2X,' field capacity cm3/cm3: ',F6.3,
     # ' wilting point: ',F7.4,
     # ' maximum plant available soil water storage, cm: ',F6.1)

C      maximum steady-state capillary rise from groundwater, cm/d

      qmax=1.
      if (GW .gt.RD+1.D+00) qmax=DMIN1(1.D+00,p1*(GW-RD)**p2)
      write (*,705) qmax
  705 format(2x,' qmax= ',SE8.3,' cm/d',/)

C     Prediction of actual evapotranspiration Eact

      if (Epot .gt. 1.15*Rain .and. qmax .gt.0.01 ) then !arid conditions
        print *,' '
        print *,' Hydropedotransfer function used '
        print *,'   '
        EER=0.05358+0.016856*SRain+0.70125*qmax**0.90303+0.0099135*Wa
        Eact=Epot - SEpot*(1.-EER)
        Eact=Dmin1(Eact,Epot)

      else                    ! humid conditions, Bagrov method used
C     ---- transfer function to estimate the Bagrov coefficient   --
        print *,' Bagrov method used '
        print * ,'     '
C       Transfer function:
        bc=0.27034*Wa**1.0478+3.7055*(dexp(1.6183*qmax)-1.)+0.2484*Cs
C
C     ----   Anaerobiosis     -----
        print *,' Severe anaerobiosis expected? (0,1)'
        ny=0
        read (*,*) ny
        if (ny == 1) then
          S=(ths-0.05-thr)/(ths-thr)
          ha=1./alpha*(S**(-1./gm)-1.)**(1./gn)
          write (*,702) ha
  702     format(/,2X,'Anaerobiosis expected at ',F5.1,
     #    ' cm pressure head')
          bc=bc-0.05794*(ha+0.27405)*qmax**0.5354-11.832*fc
        end if

        bc=DMIN1(10.D+00,bc)
        bc=DMAX1(0.5D+00,bc)
        write (*,703) bc
  703   format(/,2X,'Bagrov coefficient = ',F6.3)

        call Newton (bc, Rain, Epot, Eact )  ! solution of Bagrovs equation
      end if

C     Final, output and return

      GWR=Rain-Eact
      write (*,701)  Eact, GWR
  701 format( 2X,/,' Actual evapotranspiration: ',F7.2,' cm',5X,
     #              'Groundwater recharge ',      f7.2,' cm')
    
      ny=0
      print *,' Next soil, same climate? (0,1)'
      read (*,*) ny
      if (ny .eq. 1) goto 2
      print *,' Next case?(0,1)'
      ny=0
      read (*,*) ny
      if (ny == 1) goto 1

      print *,'done '
      stop
      end

 
      
      Subroutine Newton(bc,Rain,Epot,Eact) 
C     ************************************
C     Find Eact for given parameter bc

      implicit real*8 (a-h,o-z)

C     --- regression equation to improve initial estimate ----
      f=-0.00406+0.6198*dsqrt(Rain/Epot)+0.0495*bc
      f=DMIN1(f,0.99D+00)
      f=DMAX1(f,0.3D+00)
      xi=f*Epot ! initial estimate of Eact  

      if (bc .ge. 9.5 .or. Rain .ge. 2.D00*Epot) then
        xn=0.998*Epot
        goto 184
      end if
        
      dy=Bagrov(xi, Epot, bc)  ! calculates slope dP/dE
     
      call Simpson (Epot, xi, bc, P)
      y=P-RAIN    ! P = integrated precipitation, RAIN=observed rain 
      it=0

  182 it=it+1
      dx=0.5*(y/dy)         
      xn=xi-dx   ! xi = previous Eact, xn = new Eact
      if (xn < 0.6 *Epot)  dx=dx/2.
      if (xn > 0.95*Epot)  dx=dx/5.
 
C     -----     test:     -----      
      if (dabs(P-RAIN) < 0.0001) goto 184 ! happy finish
      
      call Simpson (Epot,xn,bc,P)
      y=P-Rain
      dy=Bagrov(xn, Epot, bc)
      xi=xn
      
      if (it > 500) then
       print *,' Newton no convergence '
       print *,'  bc= :',  bc
       stop
      end if
      goto 182
       
  184 continue
      Eact=DMIN1(xn,Epot)
      Eact=DMIN1(Eact,Rain)
  100 continue
      return
      end
      


      SUBROUTINE SIMPSON (Epot,TOP,bc,S)
C     --------------------------------------- 
      IMPLICIT REAL*8 (A-H,O-Z)

      N=200
      bottom=0.
      de=TOP/N  ! increment
      E=0.      ! E is the variable of integration
      
      y=0.     
      S=y       ! sum
      DO 301 I=N/2,1,-1
         E=E+de     
         y=Bagrov(E, Epot, bc)
         S=S+4.0*y
         E=E+de
         y=Bagrov(E, Epot, bc)
         S=S+y+y
  301 CONTINUE

      E=TOP
      y=Bagrov(E, Epot, bc)
      S=1./3.*de*(S-y)
      RETURN
      END
      
      
      Function Bagrov(E,Epot,bc)
C     -------------------------------
C     Inner function of the Bagrov equation,
C     used for numerical integration and
C     to calculate the slope dP/dE
      implicit real*8 (a-h,o-z)
      h=DMIN1(E/Epot,0.9999D+00)
      Bagrov=1./(1.-h**bc) 
      return
      end

      Subroutine VANGEN (thr,ths,alpha,gn,h,theta)
C     ********************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      theta=thr+(ths-thr)/(1.+(alpha*h)**gn)**(1.-1./gn)
      return
      end
      
      Block data
C     ----------
      IMPLICIT REAL*8 (A-H,O-Z)
      common /soils/ soilp(170),flux(68)
      data soilp
     # /1.0, 0.0,     0.3879, 0.26437, 1.35154,   
     # 2.0,  0.0,     0.3949, 0.11647, 1.25425,     
     # 3.0,  0.0519,  0.3952, 0.07097, 1.35096,  
     # 4.0,  0.0,     0.4101, 0.10486, 1.18427,     
     # 5.0,  0.0,     0.4138, 0.08165, 1.17695,     
     # 6.0,  0.0,     0.4049, 0.48458, 1.18828,     
     # 7.0,  0.0,     0.4214, 0.18023, 1.13230,     
     # 8.0,  0.0,     0.3786, 0.20387, 1.23473,     
     # 9.0,  0.0,     0.3765, 0.08862, 1.21398,     
     # 10.0, 0.0,     0.3839, 0.06005, 1.22228,    
     # 11.0, 0.1406,  0.4148, 0.04052, 1.32416, 
     # 12.0, 0.07284, 0.4091, 0.06835, 1.20501,
     # 13.0, 0.04630, 0.4129, 0.09955, 1.18213,
     # 14.0, 0.1492,  0.4380, 0.07013, 1.24572, 
     # 15.0, 0.1629,  0.4530, 0.04947, 1.17003, 
     # 16.0, 0.1154,  0.4325, 0.03401, 1.19442, 
     # 17.0, 0.0534,  0.4284, 0.04321, 1.16518, 
     # 18.0, 0.0,     0.4030, 0.01420, 1.21344,
     # 19.0, 0.0,     0.4003, 0.02513, 1.19338,    
     # 20.0, 0.0,     0.3946, 0.02747, 1.22393,    
     # 21.0, 0.0101,  0.4001, 0.01868, 1.22068, 
     # 22.0, 0.0053,  0.4031, 0.01679, 1.20668, 
     # 23.0, 0.0276,  0.4162, 0.01697, 1.20483, 
     # 24.0, 0.0,     0.5238, 0.06612, 1.05215,    
     # 25.0, 0.0,     0.4931, 0.07339, 1.06254,    
     # 26.0, 0.0,     0.49017,0.07242, 1.06062,   
     # 27.0, 0.0,     0.4589, 0.05500, 1.08166,      
     # 28.0, 0.0170,  0.4372, 0.04538, 1.12109, 
     # 29.0, 0.0,     0.4836, 0.08402, 1.07669,    
     # 30.0, 0.07841, 0.4374, 0.06194, 1.14565,
     # 31.0, 0.0,     0.4355, 0.20919, 1.11419,    
     # 32.0, 0.0,     0.4095, 0.15041, 1.33576,    
     # 33.0, 0.0,     0.3886, 0.26188, 1.35330,    
     # 34.0, 0.0,     0.37676,0.22065, 1.46574 /

      data flux
     #/1.5244E+03,-2.4467,
     # 1.8344E+03,-2.3827,
     # 5.8747E+03,-2.5284,
     # 5.1832E+02,-1.7925,
     # 5.6573E+02,-1.7205,
     # 3.9707E+02,-1.4971,
     # 2.2651E+02,-1.8035,
     # 5.8304E+02,-1.7590,
     # 8.5042E+02,-1.7423,
     # 1.2992E+03,-1.7361,
     # 1.4856E+03,-1.5862,
     # 9.7394E+02,-1.7943,
     # 7.2012E+02,-1.7661,
     # 7.6150E+02,-1.7619,
     # 3.8861E+02,-1.6707,
     # 2.2340E+03,-2.1450,
     # 1.1664E+03,-1.8089,
     # 9.6900E+03,-2.0996,
     # 2.7659E+03,-1.9182,
     # 1.9477E+03,-1.8381,
     # 3.8354E+03,-1.9891,
     # 3.7122E+03,-1.9858,
     # 3.0780E+03,-2.0077,
     # 6.2126E+01,-1.8051,
     # 9.9199E+01,-1.8691,
     # 9.8138E+01,-1.8643,
     # 2.2582E+02,-1.9140,
     # 6.2455E+02,-2.0077,
     # 2.2292E+02,-1.9629,
     # 8.5734E+02,-2.1027,
     # 2.0702E+02,-1.5199,
     # 3.0197E+03,-2.4811,
     # 1.5653E+03,-2.4537,
     # 1.6850E+04,-3.4299 /
      end

   
      
